
using System;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration;
using System.Security.Cryptography;
using System.Security.Principal;
using System.Text;
using System.Web;
using System.Xml;
using System.Data;
using System.Data.SqlClient;

namespace System.Web.Security
{
  public class DigestAuthenticationModule: IHttpModule
  {

    public virtual void Dispose()
    {
    }

    public virtual void Init(HttpApplication application)
    {
      application.AuthenticateRequest += new EventHandler(this.OnAuthenticateRequest);
      application.EndRequest += new EventHandler(this.OnEndRequest);
    }

    public void OnAuthenticateRequest(object source, EventArgs eventArgs)
    {
      HttpApplication httpApplication = (HttpApplication)source;
      string authStr = httpApplication.Request.Headers["Authorization"];
      if (authStr != null && authStr.Length != 0)
      {
        authStr = authStr.Trim();
        if (authStr.IndexOf("Digest", 0) == 0)
        {
          authStr = authStr.Substring(7);
          ListDictionary listDictionary = new ListDictionary();
          string[] elems = authStr.Split(new char[]{','});
		 string str3=String.Empty;
	    
          for (int i = 0; i < (int)elems.Length; i++)
          {
            string[] strs2 = elems[i].Split(new char[]{'='}, 2);
             str3 = strs2[0].Trim(new char[]{' ', '\"'});
            string str4 = strs2[1].Trim(new char[]{' ', '\"'});
            listDictionary.Add(str3, str4);
          }
          string str5 = (String)listDictionary["username"];
          string str6 = "";
		  string[] strs3;
          if (!GetPasswordAndRoles(httpApplication, str5, out str6, out strs3))
          {
            DenyAccess(httpApplication);
          }
          else
          {
            string authStr2;

            string str7 = ConfigurationSettings.AppSettings["System.Web.Security.DigestAuthenticationModule_Realm"];
            string str8 = String.Format("{0}:{1}:{2}", (String)listDictionary["username"], str7, str6);
            string str9 = GetMD5HashBinHex(str8);
            string authStr0 = String.Format("{0}:{1}", httpApplication.Request.HttpMethod, (String)listDictionary["uri"]);
            string authStr1 = GetMD5HashBinHex(authStr0);
            if (listDictionary["qop"] != null)
            {
              authStr2 = String.Format("{0}:{1}:{2}:{3}:{4}:{5}", new object[]{str9, (String)listDictionary["nonce"], (String)listDictionary["nc"], (String)listDictionary["cnonce"], (String)listDictionary["qop"], authStr1});
            }
            else
            {
              authStr2 = String.Format("{0}:{1}:{2}", str9, (String)listDictionary["nonce"], authStr1);
            }
            string authStr3 = GetMD5HashBinHex(authStr2);
            bool flag2 = IsValidNonce((String)listDictionary["nonce"]) == false;
            httpApplication.Context.Items["staleNonce"] = flag2;
            if ((String)listDictionary["response"] == authStr3 && !flag2)
            {
              httpApplication.Context.User = new GenericPrincipal(new GenericIdentity(str5, "Rassoc.Samples.Digest"), strs3);
            }
            else
            {
              DenyAccess(httpApplication);
            }
          }
        }
      }
    }

    public void OnEndRequest(object source, EventArgs eventArgs)
    {
      HttpApplication httpApplication = (HttpApplication)source;
      if (httpApplication.Response.StatusCode == 401)
      {
        string authStr = ConfigurationSettings.AppSettings["System.Web.Security.DigestAuthenticationModule_Realm"];
        string str2 = GetCurrentNonce();
        bool flag = false;
        object local = httpApplication.Context.Items["staleNonce"];
        if (local != null)
        {
          flag = (bool)local;
        }
        StringBuilder stringBuilder = new StringBuilder("Digest");
        stringBuilder.Append(" realm=\"");
        stringBuilder.Append(authStr);
        stringBuilder.Append("\"");
        stringBuilder.Append(", nonce=\"");
        stringBuilder.Append(str2);
        stringBuilder.Append("\"");
        stringBuilder.Append(", opaque=\"0000000000000000\"");
        stringBuilder.Append(", stale=");
        stringBuilder.Append(!flag ? "false" : "true");
        stringBuilder.Append(", algorithm=MD5");
        stringBuilder.Append(", qop=\"auth\"");
        httpApplication.Response.AppendHeader("WWW-Authenticate", stringBuilder.ToString());
        httpApplication.Response.StatusCode = 401;
      }
    }

    private void DenyAccess(HttpApplication app)
    {
      app.Response.StatusCode = 401;
      app.Response.StatusDescription = "Access Denied";
      app.Response.Write("401 Access Denied");
      app.CompleteRequest();
    }

    private string GetMD5HashBinHex(string val)
    {
      Encoding encoding = new ASCIIEncoding();
      byte[] bs = new MD5CryptoServiceProvider().ComputeHash(encoding.GetBytes(val));
      string authStr = "";
      for (int i = 0; i < 16; i++)
      {
        authStr = String.Concat(authStr, String.Format("{0:x02}", bs[i]));
      }
      return authStr;
    }

    protected virtual string GetCurrentNonce()
    {
		DateTime nonceTime = DateTime.Now + TimeSpan.FromMinutes(1);
		string expireStr = nonceTime.ToString("G");

		Encoding enc = new ASCIIEncoding();
		byte[] expireBytes = enc.GetBytes(expireStr);
		string nonce = Convert.ToBase64String(expireBytes);

		// nonce can't end in '=', so trim them from the end
		nonce = nonce.TrimEnd(new Char[] {'='});
		return nonce;
    }

    protected virtual bool IsValidNonce(string nonce)
    {
      DateTime dateTime;

      int i = nonce.Length % 4;
      if (i > 0)
      {
        i = 4 - i;
      }
      string authStr = nonce.PadRight(nonce.Length + i, '=');
	 bool flag = false;
      try
      {
        byte[] bs = Convert.FromBase64String(authStr);
        dateTime = DateTime.Parse(new ASCIIEncoding().GetString(bs));
      }
      catch (FormatException)
      {
       
        return flag;
      }
      flag = DateTime.Now <= dateTime;
      return flag;
    }

    protected virtual bool GetPasswordAndRoles(HttpApplication app, string username, out string password, out string[] roles)
    {    
      password = null;
	// database code starts here
	string strConn=ConfigurationSettings.AppSettings["dbConn"].ToString();
	SqlConnection cn = new SqlConnection(strConn);
	cn.Open();

	SqlCommand cmd = new SqlCommand();
	cmd.Connection=cn;
	cmd.CommandType=CommandType.StoredProcedure;
	cmd.CommandText="AuthenticateUser";
	cmd.Parameters.Add(new SqlParameter("@username",username));
	SqlDataReader rdr =cmd.ExecuteReader(CommandBehavior.CloseConnection);
	rdr.Read();
	password=rdr.GetString(0);
	string tmpRoles=rdr.GetString(1);
	rdr.Close();
	bool flag=false;
    roles = tmpRoles.Split('|');	
		  if (roles.Length > 0)
		  {
        flag = true;
      }
      else
      {
        flag = false;
      }
      return flag;
    }
  }

}
